/*
  Driver for Am2302/DHT11 humidity/temperature sensor
  Am2302 returns 2 bytes of humidity times 10,
  2 bytes of Celcius temperature times 10 (signed magnitude)
  2 byte of checksum (arithmetic sum of first four bytes 
*/

#include "simpletools.h"                      // Include simple tools
#define Am2303Pin 1                           // Connected to Am2303, plus a pullup

long myLong;
unsigned char Am2303Data[5];
unsigned int RelHum, RawTemp, DegreesF, DegreesC;  //humidity, temperatures
unsigned char myChecksum;                     //calculated checksum
unsigned char negCelcius;                    //below freezing 

int main()                                    // Main function
{
  // no startup code
 
  while(1)
  {  
    readAm2303(Am2303Pin);         //function returns 5 bytes in Am2303Data[4]  
    putStr("Raw Am2303 Data:  ");  // display in hex for diagnostic
    for (int i = 0; i < 5; i++) {
       putHexLen(Am2303Data[i], 2);
     }
     
    myChecksum = 0;                //prepare to calculate and check checksum 
    for(int i = 0; i < 4; i++){
       myChecksum += Am2303Data[i];  //just first 4 bytes
     }
    if (myChecksum != Am2303Data[4]) {
        putStr("Checksum Incorrect:  Calculated: ");
        putHexLen(myChecksum, 2);
        putStr("    Expected:  ");
        putHexLen(Am2303Data[4], 2);
      }
                      
    RelHum = (Am2303Data[0] << 8) + Am2303Data[1];         //sort out returned data
    RelHum = (RelHum + 5) / 10;                            //round and divide by 10
    putStr("   Rel Humidity= ");                           //and display  
    putDecLen(RelHum,2);
    

    RawTemp = (Am2303Data[2] << 8) + Am2303Data[3];       //and now temperature
    if ((RawTemp & 0x8000) == 0) {                         //at or above freezing
      negCelcius = 0;
      DegreesC = RawTemp;      
      DegreesF = ((DegreesC * 9) / 5) + 320;
    }
    else {
      negCelcius = 1;                                    //below freezing
      DegreesC = RawTemp & 0x7FFF;                       //drop sign
      DegreesF = 320 - ((DegreesC * 9) / 5);             //works only down to 0 F
    } 
    
    DegreesC = ((DegreesC) + 5) / 10;                    //round and divide by ten
    putStr("   DegreesC= ");                             //display it 
    if (negCelcius == 1) {
      putStr("-");
    }      
    putDec(DegreesC); 
                                                    
    DegreesF = (DegreesF + 5) / 10;                      //and now Fahrenheit
    putStr("   DegreesF= ");
    putDec(DegreesF);
                
    putChar(NL);
    pause(5000);    
  }                                                     //end of while(1) 
}                                                       //end of main


void readAm2303(unsigned char myPin) {
  long u50 = (CLKFREQ/1000000) * 50;     //clock tics in 50 microseconds
  long amMask = 1 << myPin;              //mask for waitPEQ instructions
  set_output(myPin, 1);                  //make it start out high
  set_direction(myPin, 1);               //make it output high
  set_output(myPin, 0);                  //now zero
  pause(1);                              //delay 1 mSec
  set_output(myPin, 1);                  //high
  set_direction(myPin, 0);               //and now an input
  waitpeq(0, amMask);                     //wait for am2303 to take it low
  waitpeq(amMask, amMask);                //and then high
  waitpeq(0, amMask);                     //and back low
                                          //we are now in the low period prior to the 
                                          //first bit of the first byte
  for(int i = 0; i < 5; i++){             //we will collect and store 5 bytes
    char theByte = 0;                     //initialize each byte
    for(int j = 7; j > -1; j--){          //8 bits per byte
      waitpeq(amMask, amMask);            //wait for beginning of high period
      unsigned long myCount = CNT;        //cnt at beginning
      waitpeq(0, amMask);                 //wait for end of high period
      if((CNT - myCount) > u50) {         //compare high period to 50 usec  
        theByte = theByte | (1 << j);     //insert a one if needful
      }                                   
    }                                     //continue with this byte
    Am2303Data[i] = theByte;              //store this byte
  }                                       //all five bytes                                 
}                                         //end of function  